'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.updatePublishPackageConfig = exports.uncompressNodeModules = exports.restorePackageConfig = exports.publishModule = exports.installProductionModules = exports.compressNodeModules = exports.backupPackageConfig = undefined;

var _trimRight2 = require('lodash/string/trimRight');

var _trimRight3 = _interopRequireDefault(_trimRight2);

var _package = require('./../package.json');

var _package2 = _interopRequireDefault(_package);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _paths = require('./paths');

var _tarFs = require('tar-fs');

var _tarFs2 = _interopRequireDefault(_tarFs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var backupPackageConfig = undefined,
    compressNodeModules = undefined,
    fileExists = undefined,
    installProductionModules = undefined,
    publishModule = undefined,
    restorePackageConfig = undefined,
    uncompressNodeModules = undefined,
    updatePublishPackageConfig = undefined;

fileExists = function (filePath) {
    try {
        _fs2.default.statSync(filePath);

        return true;
    } catch (error) {
        return false;
    }
};

exports.compressNodeModules = compressNodeModules = function (done) {
    var stream = undefined;

    stream = _fs2.default.createWriteStream(_paths.bundledModulesTarPath);

    _tarFs2.default.pack(_paths.nodeModulesPath).pipe(stream);

    stream.on('finish', function () {
        done();
    });
};

exports.uncompressNodeModules = uncompressNodeModules = function (done) {
    var stream = undefined;

    stream = _fs2.default.createReadStream('./node_modules.tar');

    _tarFs2.default.extract('./node_modules').pipe(stream);

    stream.on('finish', function () {
        done();
    });
};

exports.backupPackageConfig = backupPackageConfig = function () {
    if (!fileExists(_paths.packagePath)) {
        throw new Error('./' + _path2.default.basename(_paths.packagePath) + ' does not exist.');
    }

    if (fileExists(_paths.packageBackupPath)) {
        throw new Error('./' + _path2.default.basename(_paths.packageBackupPath) + ' already exists.');
    }

    _fsExtra2.default.copySync(_paths.packagePath, _paths.packageBackupPath);
};

exports.restorePackageConfig = restorePackageConfig = function () {
    if (!fileExists(_paths.packageBackupPath)) {
        throw new Error('./' + _path2.default.basename(_paths.packageBackupPath) + ' does not exist.');
    }

    _fsExtra2.default.copySync(_paths.packageBackupPath, _paths.packagePath);

    _fsExtra2.default.removeSync(_paths.packageBackupPath);
};

exports.updatePublishPackageConfig = updatePublishPackageConfig = function () {
    var packageConfig = undefined;

    packageConfig = _fsExtra2.default.readJsonSync(_paths.packagePath);

    packageConfig.scripts = packageConfig.scripts || {};
    packageConfig.scripts.postinstall = packageConfig.scripts.postinstall || '';

    if (packageConfig.scripts.postinstall) {
        packageConfig.scripts.postinstall = (0, _trimRight3.default)(packageConfig.scripts.postinstall, '; ');
        packageConfig.scripts.postinstall += '; ';
    }

    packageConfig.scripts.postinstall += 'rm -fr ./node_modules; mv ./bundled_modules ./node_modules';

    packageConfig.dependencies = {
        'bundle-dependencies': _package2.default.version
    };
    packageConfig.devDependencies = {};

    _fsExtra2.default.writeJsonSync(_paths.packagePath, packageConfig);
};

exports.publishModule = publishModule = function () {
    return (0, _child_process.execSync)('npm publish', {
        cwd: process.cwd(),
        encoding: 'utf8'
    });
};

exports.installProductionModules = installProductionModules = function () {
    return (0, _child_process.execSync)('npm install --production', {
        cwd: process.cwd(),
        encoding: 'utf8'
    });
};

exports.backupPackageConfig = backupPackageConfig;
exports.compressNodeModules = compressNodeModules;
exports.installProductionModules = installProductionModules;
exports.publishModule = publishModule;
exports.restorePackageConfig = restorePackageConfig;
exports.uncompressNodeModules = uncompressNodeModules;
exports.updatePublishPackageConfig = updatePublishPackageConfig;
//# sourceMappingURL=utils.js.map
